within Buildings.BoundaryConditions.SolarGeometry.BaseClasses;
block SolarAzimuth "Solar azimuth"
  extends Modelica.Blocks.Interfaces.BlockIcon;
public
  parameter Modelica.SIunits.Angle lat "Latitude";
  Modelica.Blocks.Interfaces.RealInput zen(quantity="Angle", unit="rad")
    "Zenith angle"
    annotation (Placement(transformation(extent={{-140,40},{-100,80}})));
  Modelica.Blocks.Interfaces.RealInput solTim(quantity="Time", unit="s")
    "Solar time" annotation (Placement(transformation(extent={{-140,-80},{-100,
            -40}}), iconTransformation(extent={{-140,-80},{-100,-40}})));
  Modelica.Blocks.Interfaces.RealOutput solAzi(
    final quantity="Angle",
    final unit="rad",
    displayUnit="deg") "Solar Azimuth"
    annotation (Placement(transformation(extent={{100,-10},{120,10}})));
  Modelica.Blocks.Interfaces.RealInput decAng(quantity="Angle", unit="rad")
    "Decline angle"
    annotation (Placement(transformation(extent={{-140,-20},{-100,20}})));

protected
Real arg "cos(solAzi) after data validity check";
Real tmp "cos(solAzi) before data validity check";
constant Modelica.SIunits.Time day=86400 "Number of seconds in a day";

algorithm
  tmp :=(Modelica.Math.sin(lat)*Modelica.Math.cos(zen) - Modelica.Math.sin(
    decAng))/(Modelica.Math.cos(lat)*Modelica.Math.sin(zen));

  arg :=min(1.0, max(-1.0, tmp));

  if solTim - integer(solTim/day)*day < 43200 then
    solAzi :=-Modelica.Math.acos(arg); // Negative angle if it is in the morning
  else
    solAzi := Modelica.Math.acos(arg); // Positive angle if it is in the afternoon
  end if "(A4.9a and b)";

  annotation (
    defaultComponentName="solAzi",
    Documentation(info="<html>
<p>
This component computes the solar azimuth angle.
</p>
</html>
", revisions="<html>
<ul>
<li>
Feburary 28, 2012, by Wangda Zuo:<br>
Add solar time convertion since it is removed from solTim.
</li>
<li>
May 18, 2010, by Wangda Zuo:<br>
First implementation.
</li>
</ul>
</html>"),
    Diagram(coordinateSystem(preserveAspectRatio=true,extent={{-100,-100},{100,100}}),
                    graphics),
    Icon(coordinateSystem(preserveAspectRatio=true, extent={{-100,-100},{100,
            100}}), graphics={Text(
          extent={{-150,110},{150,150}},
          textString="%name",
          lineColor={0,0,255}),
        Text(
          extent={{-100,68},{-58,58}},
          lineColor={0,0,127},
          textString="zen"),
        Text(
          extent={{-102,-54},{-60,-64}},
          lineColor={0,0,127},
          textString="solTim"),
        Text(
          extent={{-102,6},{-60,-4}},
          lineColor={0,0,127},
          textString="decAng"),
        Bitmap(extent={{-90,90},{90,-92}}, fileName=
              "modelica://Buildings/Resources/Images/BoundaryConditions/SolarGeometry/BaseClasses/SolarAzimuth.png")}),
    Icon(graphics),
    Diagram(graphics));
end SolarAzimuth;
